'use strict';

module.exports = function buildDocumentation(name, ast) {
  var documentation = [];

  documentation.push('/* RAML Client API calls */');
  documentation.push('declare var ' + name + ': {');
  documentation.push('  /**');
  documentation.push('   * Make an API request to a custom URL.');
  documentation.push('   *');
  documentation.push('   * @param {string} url - Provide a url relative to the base URI.');
  documentation.push('   * @param {any} data - Provide a data object to replace template tags in the url.');
  documentation.push('   */');
  documentation.push('  (url: string, data?): {');
  documentation.push('    delete(body?: any, options?: any, async?: boolean): any,');
  documentation.push('    head(query?: any, options?: any, async?: boolean): any,');
  documentation.push('    get(query?: any, options?: any, async?: boolean): any,');
  documentation.push('    patch(body?: any, options?: any, async?: boolean): any,');
  documentation.push('    post(body?: any, options?: any, async?: boolean): any,');
  documentation.push('    put(body?: any, options?: any, async?: boolean): any');
  documentation.push('  };');

  buildMethods(ast.methods, documentation, 1);
  buildResources(ast.resources, documentation, 1);

  documentation.push('};');

  return documentation.join('\n');
};

function buildResources(resources, documentation, identation) {
  if (resources) {
    Object.keys(resources).forEach(function (resource) {
      if (resource.startsWith('{')) {
        var methodName = resource.substring(1, resource.length - 1);

        if (methodName[0] === '+') {
          methodName = methodName.substring(1);
        }

        documentation.push('' + buildIdentation(identation) + methodName + '(' + methodName + ': string): {');

        buildMethods(resources[resource].methods, documentation, identation + 1);
        buildResources(resources[resource].resources, documentation, identation + 1);

        documentation.push(buildIdentation(identation) + '};');
      } else {
        documentation.push('' + buildIdentation(identation) + resource + ': {');

        buildMethods(resources[resource].methods, documentation, identation + 1);
        buildResources(resources[resource].resources, documentation, identation + 1);

        documentation.push(buildIdentation(identation) + '};');
      }
    });

    return documentation;
  }

  return documentation;
}

function buildMethods(methods, documentation, identation) {
  if (methods) {
    Object.keys(methods).forEach(function (method) {
      if (methods[method].queryParameters) {
        var params = Object.keys(methods[method].queryParameters);
        documentation.push(buildIdentation(identation) + '/**');
        documentation.push(buildIdentation(identation) + ' * ' + methods[method].description);
        documentation.push(buildIdentation(identation) + ' *');
        params.forEach(function (param) {
          var info = methods[method].queryParameters[param];

          documentation.push(buildIdentation(identation) + ' * @param {' + getType(info) + '} ' + info.name + (info.required ? '?' : '') + ' - ' + info.description);
        });
        documentation.push(buildIdentation(identation) + '*/');
        documentation.push('' + buildIdentation(identation) + method + '(query?: {');
        documentation.push('' + buildIdentation(identation + 1) + params.map(function (param) {
          var info = methods[method].queryParameters[param];

          return info.name + ': ' + getType(info);
        }).join(',\n' + buildIdentation(identation + 1)));
        documentation.push(buildIdentation(identation) + '}, options?: { proxy: string, beforeSend: () => void, headers: object, body: object, baseUriParameters: object }): any;');
      } else {
        documentation.push('' + buildIdentation(identation) + method + '(' + getMethodInfo(method) + '): any;');
      }
    });

    return documentation;
  }

  return documentation;
}

function getMethodInfo(method) {
  var usesQuery = ['get', 'head'];

  if (usesQuery.indexOf(method) > -1) {
    return 'query?: any, options?: any, async?: boolean';
  }

  return 'body?: any, options?: any, async?: boolean';
}

function getType(info) {
  if (info.type && info.type[0]) {
    if (info.type[0] === 'integer') {
      return 'number';
    }

    if (['number', 'boolean', 'string', 'any'].some(function (type) {
      return type === info.type[0];
    })) {
      return info.type[0];
    }
  }

  return 'any';
}

function buildIdentation(identation) {
  return new Array(identation + 1).join('  ');
}
//# sourceMappingURL=documentation.js.map